<?php     
namespace Concrete\Package\MoceanApiBroadcast\Controller\SinglePage\Dashboard\MoceanApiBroadcast;

use \Datetime;
use \Concrete\Core\Package\PackageService;
use \Concrete\Core\Http\Request;
use \Concrete\Core\Page\Controller\DashboardPageController;
use \Concrete\Package\MoceanApiBroadcast\Src\Entity\MoceanHistory;

class Send extends DashboardPageController
{
    public function view()
    {
        //set the form settings
        $this->setFormSettings();
    }
    
    public function submitBroadcast()
    {
        $this->post = Request::post();
        if(!empty($this->post))
        {
            $pkg = $this->app->make(PackageService::class)->getByHandle('mocean_api_broadcast');
            $phoneAttributeHandle = $pkg->getConfig()->get('mocean_api_config.phone_attribute_handle');
            
            $message = $this->post['message'];
            $selected = $this->post['selectRecipient'];
            
            $count = 0;
            $fail = 0;
            $error = false;
            
            switch ($selected) 
            {
                //no valid recipient selected
                case 0:
                    $this->error->add('No recipient selected.');
                    $this->setFormSettings();
                    break;
                //all users
                case 1:
                    $list = new \Concrete\Core\User\UserList();
                    $users = $list->getResults();
                    foreach($users as $user) {
                        $recipient = $user->getAttribute($phoneAttributeHandle);
                        if($recipient) 
                        {
                            $count++;
                            $recipient = preg_replace("/[^0-9]/", "", $recipient);
                            $validRecipient = $this->processCountryCode($recipient);
                            $invalid = true;
                            if($validRecipient)
                            {
                                $invalid = false;
                                $recipient = $validRecipient;
                            }
                            if($this->sendSms($message, $recipient, $invalid))
                            {
                                $error = true;
                                $fail++;
                            }
                        }
                    }
                    break;
                //specific groups
                case 2:
                    //no valid group selected
                    if(!$this->post['specificGroup'])
                    {
                        $this->error->add('No group selected.');
                    }
                    else
                    {
                        if($this->post['specificGroup'] == 1)
                        {
                            $list = new \Concrete\Core\User\Group\GroupList();
                            $groups = $list->getResults();
                            
                        }
                        else
                        {
                            $app = \Concrete\Core\Support\Facade\Application::getFacadeApplication();
                            $groupId = $app->make(\Concrete\Core\Tree\Node\Type\Group::class)->getByID($this->post['specificGroup'])->getTreeNodeGroupID();
                            $groups = array();
                            $groups[] = $app->make(\Concrete\Core\User\Group\Group::class)->getByID($groupId);
                        }
                        
                        foreach($groups as $group) 
                        {
                            $users = $group->getGroupMemberIDs();
                            foreach($users as $uid) {
                                $app = \Concrete\Core\Support\Facade\Application::getFacadeApplication();
                                $user = $app->make(\Concrete\Core\User\UserInfoRepository::class)->getByID($uid);
                                $recipient = $user->getAttribute($phoneAttributeHandle);
                                if($recipient)
                                {
                                    $count++;
                                    $recipient = preg_replace("/[^0-9]/", "", $recipient);
                                    $validRecipient = $this->processCountryCode($recipient);
                                    $invalid = true;
                                    if($validRecipient)
                                    {
                                        $invalid = false;
                                        $recipient = $validRecipient;
                                    }
                                    if($this->sendSms($message, $recipient, $invalid))
                                    {
                                        $error = true;
                                        $fail++;
                                    }
                                }
                            }
                        }
                    }
                    break;
                //specific users
                case 3:
                    //no valid recipient selected
                    if(!$this->post['specificRecipient'])
                    {
                        $this->error->add('No recipient selected.');
                    }
                    else
                    {
                        foreach($this->post['specificRecipient'] as $uid) {
                            $app = \Concrete\Core\Support\Facade\Application::getFacadeApplication();
                            $user = $app->make(\Concrete\Core\User\UserInfoRepository::class)->getByID($uid);
                            $recipient = $user->getAttribute($phoneAttributeHandle);
                            if($recipient)
                            {
                                $count++;
                                $recipient = preg_replace("/[^0-9]/", "", $recipient);
                                $validRecipient = $this->processCountryCode($recipient);
                                $invalid = true;
                                if($validRecipient)
                                {
                                    $invalid = false;
                                    $recipient = $validRecipient;
                                }
                                if($this->sendSms($message, $recipient, $invalid))
                                {
                                    $error = true;
                                    $fail++;
                                }
                            }
                        }
                    }
                    break;
                //specific phone numbers
                case 4:
                    $specificPhone = $this->post['specificPhone'];
                    $recipients = array();
                    $recipients = preg_split('~[^0-9]~i', $specificPhone, -1, PREG_SPLIT_NO_EMPTY);
                    
                    foreach($recipients as $recipient) {
                        $count++;
                        $recipient = preg_replace("/[^0-9]/", "", $recipient);
                        $validRecipient = $this->processCountryCode($recipient);
                        $invalid = true;
                        if($validRecipient)
                        {
                            $invalid = false;
                            $recipient = $validRecipient;
                        }
                        if($this->sendSms($message, $recipient, $invalid))
                        {
                            $error = true;
                            $fail++;
                        }
                    }
                    break;
            }
            if($count == 0)
                $this->set('message', t('No phone number for selected recipient(s).'));
            else if($error)
                $this->error->add($count - $fail.' out of '.$count.' SMS sent, check SMS Transaction History for more details.');
            else
                $this->set('success', t($count.' out of '.$count.' SMS sent.'));
        }
        $this->setFormSettings();
    }
    
    public function processCountryCode($recipient) 
    {
        $pkg = $this->app->make(PackageService::class)->getByHandle('mocean_api_broadcast');
        $countryCode = $pkg->getConfig()->get('mocean_api_config.country_code');
        
        //no country selected
        if(!$countryCode || $countryCode === 0)
        {
            return $recipient;
        }
        
        //check country code
        $request = new \Mocean\Sms();
        $request->setRecipient($recipient);
        $request->setCountryCode($countryCode);
        
        $recipient = $request->checkCountryCode();
        
        return $recipient;
    }
    
    public function sendSms($message, $recipient, $invalidRecipient) 
    {
        $pkg = $this->app->make(PackageService::class)->getByHandle('mocean_api_broadcast');
        $apiKey = $pkg->getConfig()->get('mocean_api_config.api_key');
        $apiSecret = $pkg->getConfig()->get('mocean_api_config.api_secret');
        $sender = $pkg->getConfig()->get('mocean_api_config.sender');
        
        if($invalidRecipient)
        {
            $this->insertHistory($sender, $message, $recipient, "Invalid phone number for selected country", "Fail");
            return true;
        }
        
        $error = false;
        
        //send sms
        $request = new \Mocean\Sms();
        $request->setApiKey($apiKey);
        $request->setApiSecret($apiSecret);
        $request->setSender($sender);
        $request->setRecipient($recipient);
        $request->setMessage($message);
        
        //sms response
        $jsonSms = $request->sendBroadcast();
        
        if (count($jsonSms) == count($jsonSms, COUNT_RECURSIVE)) 
        {
            $errMsg = $jsonSms['err_msg'];
            $status = $jsonSms['status'];
        }
        else 
        {
            $errMsg = $jsonSms['messages'][0]['err_msg'];
            $status = $jsonSms['messages'][0]['status'];
        }
        
        if($status == 0) 
        {
            $response = 'Message sent';
            $status = 'Success';
        }
        else 
        {
            $response = $errMsg;
            $status = 'Fail';
            $error = true;
        }
        
        $this->insertHistory($sender, $message, $recipient, $response, $status);
        
        return $error;
    }
    
    public function insertHistory($sender, $message, $recipient, $response, $status) 
    {
        //insert sms transaction history
        $history = new MoceanHistory();
        
        $history->setSender($sender);
        $history->setDatetime(new DateTime());
        $history->setMessage($message);
        $history->setRecipient($recipient);
        $history->setResponse($response);
        $history->setStatus($status);
        $history->save();
    }

    public function setFormSettings()
    {
        $pkg = $this->app->make(PackageService::class)->getByHandle('mocean_api_broadcast');
        $phoneAttributeHandle = $pkg->getConfig()->get('mocean_api_config.phone_attribute_handle');
        //recipient choices
        $selectRecipient = array(0 => t('-- Select Recipient --'), 1 => t('All Users'), 2 => t('Specific Group'), 3 => t('Specific Users'), 4 => t('Specific Phone Numbers'));
        $this->set('selectRecipient', $selectRecipient);
    }
}
?>
